@@ -1,5 +1,5 @@
 /*
- * $Id: TestSuite.java,v 1.1.1.1 1998/05/11 18:43:03 metlov Exp $
+ * $Id: TestSuite.java,v 1.5 1998/05/25 15:51:47 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -17,6 +17,7 @@
 import gnu.jel.*;
 import gnu.jel.generated.EC;
 import gnu.jel.generated.ParseException;
+import gnu.jel.generated.TokenMgrError;
 import java.io.PrintStream;
 
 public class TestSuite {
@@ -45,30 +46,39 @@ public class TestSuite {
     o.println(); o.println("----- TESTING ERROR REPORTING ------");
     o.println();
     
-    testError("1.0-+1.0",lib,5,o);
-    testError("1.0&1.0",lib,4,o);
-    testError("-",lib,1,o);
-    testError("0x56+0xXX",lib,7,o);
-    testError("Sin(x)",lib,5,o);
-    testError("Sin(6)",lib,1,o);
-    testError("sin(' ')",lib,1,o);
-    testError("'a'+'b'",lib,4,o);
-    testError("1+sin(1,6)",lib,3,o);
-    testError("2147483649L+2147483649",lib,13,o);
-    testError("01234567+08+5",lib,11,o);
+    testError("1.0-+1.0",null,lib,5,o);
+    testError("1.0&1.0",null,lib,4,o);
+    testError("-",null,lib,1,o);
+    testError("0x56+0xXX",null,lib,7,o);
+    testError("Sin(x)",null,lib,5,o);
+    testError("Sin(6)",null,lib,1,o);
+    testError("sin(' ')",null,lib,1,o);
+    testError("'a'+'b'",null,lib,4,o);
+    testError("1+sin(1,6)",null,lib,3,o);
+    testError("2147483649L+2147483649",null,lib,13,o);
+    testError("01234567+08+5",null,lib,11,o);
+    testError("0.5+0#4",null,lib,6,o);
+    testError("0.5+1",Integer.TYPE,lib,5,o);
+    testError("0.5+(floatp)0.4D",null,lib,6,o);
+    testError("0.5+(boolean)0.4D",null,lib,6,o);
     
     o.println(); o.println("----- TESTING STATIC OPTIMIZATIONS ------");
     o.println();
     
-    testExpression("2*2",new Long(4),null,lib,o);
-    testExpression("2L*2L",new Long(4),null,lib,o);
-    testExpression("2.0*2.0",new Double(4.0),null,lib,o);
-    testExpression("2.0F*2.0F",new Double(4.0),null,lib,o);
-    testExpression("sin(1)",new Double(Math.sin(1.0)),null,lib,o);
-    testExpression("pow(sin(1),2)+pow(cos(1),2)",new Double(1.0),null,lib,o);
-    testExpression("min(1+2*2,(1+2)*2)",new Long(5),null,lib,o);
+    testExpression("2*2",new Byte((byte)4),null,null,lib,o);
+    testExpression("2L*2L",new Long(4),null,null,lib,o);
+    testExpression("2.0*2.0",new Double(4.0),null,null,lib,o);
+    testExpression("2.0F*2.0F",new Float(4.0F),null,null,lib,o);
+    testExpression("sin(1)",new Double(Math.sin(1.0)),null,null,lib,o);
+    testExpression("pow(sin(1),2)+pow(cos(1),2)",new Double(1.0),null,null,
+		   lib,o);
+    testExpression("min(1+2*2,(1+2)*2)",new Integer(5),null,null,lib,o);
+    testExpression("7+4-6",new Double(5.0),Double.TYPE,null,lib,o);
+    testExpression("7+(int)4-(int)6.0+1-(int)((double)((float)((long)1+0)+0)+0)",
+		   new Integer(5),null,null,lib,o);
 
-    o.println(); o.println("----- TESTING DYNAMIC OPTIMIZATIONS (x=5)------");
+    o.println(); 
+    o.println("----- TESTING DYNAMIC OPTIMIZATIONS (x=5.0D) ------");
     o.println();
 
     // Construct a new library
@@ -86,27 +96,28 @@ public class TestSuite {
     dynamicLib[0]=vp.getClass();
     lib=new Library(staticLib,dynamicLib);
     
-    testExpression("sin(x/5)",new Double(Math.sin(1.0)),rtp,lib,o);
-    testExpression("255+5+7+9-x",new Double(255+7+9),rtp,lib,o);
-    testExpression("-x+255+5+7+9",new Double(255+7+9),rtp,lib,o);
-    testExpression("-x+(255+5+7+9)",new Double(255+7+9),rtp,lib,o);
+    testExpression("sin(x/5)",new Double(Math.sin(1.0)),null,rtp,lib,o);
+    testExpression("255+5+7+9-x",new Double(255+7+9),null,rtp,lib,o);
+    testExpression("-x+255+5+7+9",new Double(255+7+9),null,rtp,lib,o);
+    testExpression("-x+(255+5+7+9)",new Double(255+7+9),null,rtp,lib,o);
+    testExpression("5*x-66",new Double(25-66),Double.TYPE,rtp,lib,o);
+    testExpression("7+(int)4-(int)6.0+(int)x-(int)((double)((float)((long)x+1)+2)+3)+6",
+		   new Integer(5),null,rtp,lib,o);
+
     
     o.println(); o.println("----- TESTING COMPILED-in EXCEPTIONS (x=5)------");
     o.println();
     
-    testExpression("(1+6)/(2+2-4)",null,rtp,lib,o);
-    testExpression("throw_arg_eq_4(6-2)",null,rtp,lib,o);
+    testExpression("(1+6)/(2+2-4)",null,null,rtp,lib,o);
+    testExpression("throw_arg_eq_4(6-2)",null,null,rtp,lib,o);
     
     o.println(); o.println("^^^^^^^^^^^^ SCORE success/failure = "+
 			   successes+"/"+failures+
 			   " ^^^^^^^^^^^^" );
-    o.println();
-    o.println("NOTE: Error detection test \"sin(' ')\" currently "+
-	      "always fails.");
 
   };
 
-  private static void testExpression(String expr, Object tobe, 
+  private static void testExpression(String expr, Object tobe, Class fixType,
 				     Object[] runtimeParameters,
 				     Library lib,   PrintStream o ) {
     o.print("*** : \""); o.print(expr); 
@@ -117,7 +128,7 @@ public class TestSuite {
     };
     OpenOptimizer op=null;
     try {
-      op=fillOptimizer(expr,lib);
+      op=fillOptimizer(expr,lib,fixType);
     } catch (CompilationException ce) {
       o.print("--- COMPILATION ERROR :");
       o.println(ce.getMessage());
@@ -174,13 +185,13 @@ public class TestSuite {
     
   };
   
-  private static void testError(String expr, Library lib, int errcol,
-				PrintStream o) {
+  private static void testError(String expr,Class fixType, Library lib, 
+				int errcol, PrintStream o) {
     o.print("*** : \""); o.print(expr); o.println('"');
 
     CompilationException ce=null;
     try {
-      OpenOptimizer op=fillOptimizer(expr,lib);
+      OpenOptimizer op=fillOptimizer(expr,lib,fixType);
     } catch (CompilationException e) {
       ce=e;
     };
@@ -218,14 +229,16 @@ public class TestSuite {
 
 
   // Mostly copied from jel.Evaluator
-  static OpenOptimizer fillOptimizer(String expression, Library lib) 
+  static OpenOptimizer fillOptimizer(String expression, Library lib, Class rt) 
        throws CompilationException {
-    EC ec=new EC(new java.io.StringReader(expression));
+	 EC ec=new EC(new java.io.StringReader(expression));
     OpenOptimizer o=new OpenOptimizer(lib);
     try {
-      ec.expression(o);
+      ec.expression(o,rt);
     } catch (ParseException pe) {
       throw new CompilationException(pe);
+    } catch (TokenMgrError tme) {
+      throw new CompilationException(tme);
     };
     return o;
   };
