@@ -1,5 +1,5 @@
 /*
- * $Id: ExpressionImage.java,v 1.1.1.1 1998/05/11 18:43:02 metlov Exp $
+ * $Id: ExpressionImage.java,v 1.4 1998/05/25 15:51:19 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -75,11 +75,17 @@ public class ExpressionImage {
   public final static int BI_XO=7;
 
 
+  /**
+   * Names of binary operations by ID in the readable form.
+   */
   public final static String[] binaryNames={"add","substract","multiply",
 					    "divide","remainder",
 					    "bitwise and","bitwise or",
 					    "bitwise xor"};
-  
+
+  /**
+   * Names of unary operations by ID in the readable form.
+   */
   public final static String[] unaryNames={"negate"};
   /**
    * Denotes the unary NEGATION operation.
@@ -126,40 +132,19 @@ public class ExpressionImage {
     /*13*/ 0x0A, 0x00, 0x04, 0x00, 0x0C, // Mref, C CP4,N&T CP12 (super.<init>)
     /*14*/ 0x0C, 0x00, 0x05, 0x00, 0x06, // N&T, N CP5, T CP6    (Object[] e;)
     /*15*/ 0x09, 0x00, 0x02, 0x00, 0x0E, // Fref, C CP2, N&T CP14(this.e)
-    /*16*/ 0x01, 0x00, 0x08, // UTF 8 bytes "evaluate"
-	   0x65, 0x76, 0x61, 0x6C, 0x75, 0x61, 0x74, 0x65,
-    /*17*/ 0x01, 0x00, 0x27, // UTF 39 bytes "([Ljava/lang/Object;)Ljava/lang/Object;"
-	   0x28, 0x5B, 0x4C, 0x6A, 0x61, 0x76, 0x61, 0x2F, 0x6C, 0x61,
-	   0x6E, 0x67, 0x2F, 0x4F, 0x62, 0x6A, 0x65, 0x63, 0x74, 0x3B,
-	   0x29, 0x4C, 0x6A, 0x61, 0x76, 0x61, 0x2F, 0x6C, 0x61, 0x6E,
-	   0x67, 0x2F, 0x4F, 0x62, 0x6A, 0x65, 0x63, 0x74, 0x3B,
+    /*16*/ 0x01, 0x00, 0x07, // UTF 7 bytes "getType"
+	   0x67, 0x65, 0x74, 0x54, 0x79, 0x70, 0x65,
+    /*17*/ 0x01, 0x00, 0x03, // UTF 3 bytes "()I"
+	   0x28, 0x29, 0x49,
     /*18*/ 0x01, 0x00, 0x13, // UTF 19 bytes "java/lang/Throwable"
 	   0x6A, 0x61, 0x76, 0x61, 0x2F, 0x6C, 0x61, 0x6E, 0x67, 0x2F,
 	   0x54, 0x68, 0x72, 0x6F, 0x77, 0x61, 0x62, 0x6C, 0x65,
     /*19*/ 0x07, 0x00, 0x12,  // Class name in CP19 ("java/lang/Throwable");
-
-	   // method references for constructors of primitive types
-    /*20*/ 0x01, 0x00, 0x04,  // UTF 4 bytes "(D)V" ,constructor of Double
-	   0x28, 0x44, 0x29, 0x56,
-    /*21*/ 0x01, 0x00, 0x04,  // UTF 4 bytes "(J)V" ,constructor of Long
-	   0x28, 0x4A, 0x29, 0x56,
-    /*22*/ 0x0C, 0x00, 0x07, 0x00, 0x14, // N&T, N CP7, T CP21 <init>(double)
-    /*23*/ 0x0C, 0x00, 0x07, 0x00, 0x15, // N&T, N CP7, T CP20 <init>(long)
-    /*24*/ 0x01, 0x00, 0x10, // UTF 16 bytes "java/lang/Double"
-	   0x6A, 0x61, 0x76, 0x61, 0x2F, 0x6C, 0x61, 0x6E, 0x67, 0x2F,
-	   0x44, 0x6F, 0x75, 0x62, 0x6C, 0x65,
-    /*25*/ 0x01, 0x00, 0x0E, // UTF 14 bytes "java/lang/Long"
-	   0x6A, 0x61, 0x76, 0x61, 0x2F, 0x6C, 0x61, 0x6E, 0x67, 0x2F,
-	   0x4C, 0x6F, 0x6E, 0x67,
-    /*26*/ 0x07, 0x00, 0x18, // CLASS name is in CP24 , java.lang.Double
-    /*27*/ 0x07, 0x00, 0x19, // CLASS name is in CP25 , java.lang.Long
-    /*28*/ 0x0A, 0x00, 0x1a, 0x00, 0x16, //Mref,C CP26,N&T CP22 (Double.<init>)
-    /*29*/ 0x0A, 0x00, 0x1b, 0x00, 0x17, //Mref,C CP27,N&T CP23 (Long.<init>)
-    /*30*/ // DON'T FORGET TO CHANGE CONSTRUCTOR IF YOU ADD SOMETHING !!!
+    /*20*/ // DON'T FORGET TO CHANGE CONSTRUCTOR IF YOU ADD SOMETHING !!!
 	  
   };
   
-  private static final byte[] intermezzo={ 
+  private static final byte[] intermezzo1={ 
     0x00, 0x21,             // Access: PUBLIC, AC_SUPER
     0x00, 0x02,             // This class info is CP2
     0x00, 0x04,             // Super class info is CP4
@@ -169,7 +154,7 @@ public class ExpressionImage {
     /* */  0x00, 0x05,        // Name UTF is CP5, (non descriptive? who cares?)
     /* */  0x00, 0x06,        // Descriptor UTF is CP6
     /* */  0x00, 0x00,        // No attributes ( just a field)
-    0x00, 0x02,             // have two methods
+    0x00, 0x03,             // have three methods
     /*1*/  0x00, 0x01,        // PUBLIC
     /* */  0x00, 0x07,        // Name UTF is CP7, ("<init>")
     /* */  0x00, 0x08,        // Descriptor UTF is CP8
@@ -191,8 +176,31 @@ public class ExpressionImage {
     /*    */   0x00, 0x00,             // no exception table
     /*    */   0x00, 0x00,             // no code attributes (don't DEBUG ;)
     /*2*/  0x00, 0x01,        // PUBLIC
-    /* */  0x00, 0x10,        // Name UTF is CP16 ("evaluate")
-    /* */  0x00, 0x11,        // Descriptor UTF is CP17(Object evaluate(Object[])
+    /* */  0x00, 0x10,        // Name UTF is CP16, ("getType")
+    /* */  0x00, 0x11,        // Descriptor UTF is CP17 "()I"
+    /* */  0x00, 0x02,        // two attributes, CODE and exceptions
+    /*   */  0x00, 0x09,         //-- attribute name is CP9 ("Exceptions")
+    /*     */  0x00, 0x00, 0x00, 0x02, // length is 2 bytes (following)
+    /*     */  0x00, 0x00,             // No Checked Exceptions Thrown
+    /*   */  0x00, 0x0A,         //-- attribute name is CP10 ("Code")
+    /*     */  0x00, 0x00, 0x00, 0x0F, // length is 15 bytes
+    /*     */  0x00, 0x01,             // stack size is 2
+    /*     */  0x00, 0x01,             // locals number is 2
+    /*         */  0x00, 0x00, 0x00, 0x03, // the code is 3 bytes long
+    /*       00*/  0x10,                   // bipush
+  };
+  private byte thetype=99; // The actual type of the expression
+  
+  private static final byte[] intermezzo2={
+    /*       02*/  (byte)0xAC,             // ireturn
+    /*    */   0x00, 0x00,             // no exception table
+    /*    */   0x00, 0x00,             // no code attributes (don't DEBUG ;)
+    /*3*/  0x00, 0x01,        // PUBLIC (THE MAIN METHOD,WHICH DOES EVALUATION)
+  };
+  short nameIdx=9999;      // Index of the name in CP
+  short signIdx=9999;      // Index of descriptor in CP
+
+  private static final byte[] intermezzo3={
     /* */ 0x00, 0x02,        // two attributes, Exceptions and Code
     /*   */ 0x00, 0x09,         // -- attribute name in CP9 ("Exceptions")
     /*     */ 0x00, 0x00, 0x00, 0x04,   // length is 4 bytes (following)
@@ -216,25 +224,53 @@ public class ExpressionImage {
   };
   
   // FINITA
-  
-  private static Class[] 
-  primitiveIndices={Boolean.TYPE, Byte.TYPE   , Character.TYPE,
+
+  /**
+   * Classes of the primitive types by ID
+   */
+  public final static Class[] 
+  primitiveTypes={Boolean.TYPE, Byte.TYPE   , Character.TYPE,
 		    Short.TYPE  , Integer.TYPE, Long.TYPE,
-		    Float.TYPE  , Double.TYPE};
+		    Float.TYPE  , Double.TYPE, Void.TYPE };
+  
+  private final static char[] primitiveCodes= {'Z','B','C','S',
+					       'I','J','F','D','V'};
+  
+  /**
+   * Names of the primitive types by ID in readable form.
+   */
+  public final static String[] primitiveTypeNames = {
+    "boolean","byte","char" ,"short" ,
+    "int"    ,"long","float","double"
+  };
+  
+  private final static byte[] returns={
+    (byte)0xAC, (byte)0xAC, (byte)0xAC, (byte)0xAC,
+    (byte)0xAC, (byte)0xAD, (byte)0xAE, (byte)0xAF,
+  };
+
+  // All conversion tables ignore the type void completely. The object
+  // type is represented by primitiveindex == primitiveTypes.length
   
   // Primitive type conversions
   
-  // Widening conversions
-    private final static  boolean[][] 
-  //           Z     B     C     S     I     J     F     D   <-- OTKUDA
-  cvt_wide={{ true,false,false,false,false,false,false,false}, // Z
-	    { true, true,false,false,false,false,false,false}, // B
-	    { true, true, true,false,false,false,false,false}, // C
-	    { true, true, true, true,false,false,false,false}, // S
-	    { true, true, true, true, true,false,false,false}, // I
-	    { true, true, true, true, true, true,false,false}, // J
-	    { true, true, true, true, true, true, true,false}, // F
-	    { true, true, true, true, true, true, true, true}};// D
+  // Possible widening conversions
+  private final static byte[] cvt_wide= {
+    (byte)0x80,0x40,0x60,0x50,0x58,0x5C,0x5E,0x5F
+  };
+
+  // Widening conversions decoded:
+  //      Z     B     C     S     I     J     F     D   <--- OTKUDA
+  //  --------------------------------------------------
+  //      1     0     0     0     0     0     0     0  |  Z  
+  //      0     1     0     0     0     0     0     0  |  B
+  //      0     1     1     0     0     0     0     0  |  C
+  //      0     1     0     1     0     0     0     0  |  S
+  //      0     1     0     1     1     0     0     0  |  I
+  //      0     1     0     1     1     1     0     0  |  J
+  //      0     1     0     1     1     1     1     0  |  F
+  //      0     1     0     1     1     1     1     1  |  D
+  //  --------------------------------------------------
 
 
   //  OTKUDA                            Z   B   C   S   I   J   F   D 
@@ -283,7 +319,7 @@ public class ExpressionImage {
   private final static  int[] norm = {000,145,146,147,000,000,000,000};
 
   //                                   Z   B   C   S   I   J   F   D 
-  private final static int[][] una ={{116,255,255,116,116,117,118,119}};// NE
+  private final static int[][] una ={{116,116,255,116,116,117,118,119}};// NE
 
   //                                   Z   B   C   S   I   J   F   D   L
   private final static byte[] stor = { 54, 54, 54, 54, 54, 55, 56, 57, 58};
@@ -392,7 +428,7 @@ public class ExpressionImage {
 				    "Class name Should be the first in CP");
     try {    
       constPoolData.write(cp_middle);  // The rest of CP is already prepared
-      poolEntries+=28; // Number of entries in the rest
+      poolEntries+=18; // Number of entries in the rest
     } catch (java.io.IOException e) {
       if (Debug.enabled) Debug.reportThrowable(e);
     };
@@ -706,7 +742,12 @@ public class ExpressionImage {
       imageData.write(prologue);
       imageData.writeShort(poolEntries);
       constPool.writeTo(imageData);
-      imageData.write(intermezzo);
+      imageData.write(intermezzo1);   // start getType
+      imageData.write(thetype);       // put the actual type in
+      imageData.write(intermezzo2);   // start evaluate_XXX
+      imageData.writeShort(nameIdx);  
+      imageData.writeShort(signIdx);
+      imageData.write(intermezzo3);   // come to evaluate code
       int code_len=methodText.size();
       imageData.writeInt(code_len+12);
       imageData.writeShort(max_stack);
@@ -784,11 +825,11 @@ public class ExpressionImage {
       Debug.assert(c.isPrimitive(),
 		   "PrimitiveID should be used for primitive types only.");
     int i;
-    for(i=0;(i<primitiveIndices.length) && (primitiveIndices[i]!=c);i++);
+    for(i=0;(i<primitiveTypes.length) && (primitiveTypes[i]!=c);i++);
     if (Debug.enabled)
-      Debug.assert(i<primitiveIndices.length,
+      Debug.assert(i<primitiveTypes.length,
 		   "You IDIOT !!! You didn't put _ALL_ primitive types"+
-		   " into primitiveIndices array.");
+		   " into primitiveTypes array.");
     return i;
   };
 
@@ -975,7 +1016,7 @@ public class ExpressionImage {
     if (pt2 && pt1) {
       int it1=primitiveID(t1);
       int it2=primitiveID(t2);
-      return cvt_wide[it2][it1];
+      return (cvt_wide[it2] & (0x80 >> it1)) >0 ;
     };
     if (pt2 ^  pt1) return false; // Can't convert yet between PT and OBJ.
     return t2.isAssignableFrom(t1);
@@ -1045,7 +1086,7 @@ public class ExpressionImage {
     int ic2=primitiveID(c2);
     int idx=cvts[ic1][ic2];
     if (idx==255) return null;
-    return primitiveIndices[idx];
+    return primitiveTypes[idx];
   };
 
   /**
@@ -1305,58 +1346,54 @@ public class ExpressionImage {
   public void asm_return() {
     checkAlter();
     Class typeout=null;
+    boolean returnvoid=false;
     try { typeout=(Class) typesStk.peek(); } 
     catch (java.util.EmptyStackException e) {
       // Nothing in stack is the same as void in stack and is returned as null
       asm_load_object(null);
+      returnvoid=true;
       //if (Debug.enabled) Debug.reportThrowable(e);
     };
-    
-    // Convert type to object
-    if (typeout!=null) {
-      if        ((typeout==Double.TYPE) || (typeout==Float.TYPE)) {
-	asm_convert(Double.TYPE); // These two go as java.lang.Double
-	currSSW+=2; ensureStack();
-	// Let's create return(new java.lang.Double(what_is_now_in_stack));
-	codeOP(0x49);        //| dstore_2
-	codeOP(0xbb);        //| new
-	codeINDEX(26);       //|  <ind_1><ind_2> java.lang.Double
-	codeOP(0x59);        //| dup
-	codeOP(0x28);        //| dload_2
-	codeOP(0xb7);        //| invokespecial
-	codeINDEX(28);       //| <ind_1><ind_2> java.lang.Double.<init>(double)
-	//	codeOP(0xb0);        //|  areturn
-	currSSW-=3;
-	classFinished=true;      
-      } else if (typeout.isPrimitive()) { // long, int, char, byte, short, bool
-	asm_convert(Long.TYPE); // all these are returned as java.lang.Long
-	currSSW+=2; ensureStack();
 
-	// Let's create return(new java.lang.Long(what_is_now_in_stack));
-	codeOP(0x41);        //| lstore_2
-	codeOP(0xbb);        //| new
-	codeINDEX(27);       //|  <ind_1><ind_2> java.lang.Long
-	codeOP(0x59);        //| dup
-	codeOP(0x20);        //| lload_2
-	codeOP(0xb7);        //| invokespecial
-	codeINDEX(29);       //|  <ind_1><ind_2> java.lang.Long.<init>(long)
-	//	codeOP(0xb0);        //|  areturn
-	currSSW-=3;
-	classFinished=true;      
-      }; /* other primitive types should go here */ 
-      
+    // Find out the result type
+    int prid=primitiveTypes.length; // void = 8
+    byte instruction=(byte) 0xB0;         // return instruction : areturn
+    if (!returnvoid) {
+      prid++; // Object =9;
+      if ((typeout!=null) && (typeout.isPrimitive())) {
+	prid=primitiveID(typeout);
+	instruction=returns[prid];
+      };
     };
 
+    StringBuffer mname=new StringBuffer("evaluate");
+    StringBuffer signature=new StringBuffer("([Ljava/lang/Object;)");
+    // Form signature and name
+    if (prid<primitiveTypes.length) {
+      mname.append('_'); mname.append(primitiveTypeNames[prid]);
+      signature.append(primitiveCodes[prid]);
+    } else {
+      signature.append("Ljava/lang/Object;");
+    };
+    
+    // put name & signature into cp
+    nameIdx=(short)getUTFIndex(mname.toString());
+    signIdx=(short)getUTFIndex(signature.toString());
+    
+    // Mark type
+    thetype=(byte)prid;
+    
+    // Generate return instruction
     typesStk.pop();
-
+    
     if (Debug.enabled) 
       Debug.assert(typesStk.size()==0,"Stack is not empty when returning.");
-
+    
     if (Debug.enabled)
-      Debug.assert(currSSW==1,"Words left in stack = "+currSSW+"should be 1.");
-
-    // Now we certainly have the object in stack (can be null)
-    codeOP(0xb0);              //|  areturn
+      Debug.assert((prid>7) || (currSSW==stkoccup[prid]),
+		   "Words left in stack = "+currSSW+"should be 1.");
+    
+    codeOP(instruction);              //|  XXXreturn
     classFinished=true;
   };
   
@@ -1472,7 +1509,7 @@ public class ExpressionImage {
 	t.compare(sii,si);
 
 	t.startTest("Checking the index of added UTF");
-	t.compare(si,30);
+	t.compare(si,20);
 	
 	t.startTest("Add Long twice");
 	int li=ei.getLongIndex(new Long(10));
@@ -1525,8 +1562,8 @@ public class ExpressionImage {
 	};
 
 	t.startTest("Counting added entries ");
-	t.compare(ei.poolEntries,14+2+29+1);
-        // 14(entries added), 2(longs/doubles), 29(in cp_start), 1(zeroth) 
+	t.compare(ei.poolEntries,14+2+19+1);
+        // 14(entries added), 2(longs/doubles), 19(in cp_start), 1(zeroth) 
 
 	t.startTest("Loading and executing via getExpression");
 	try {
@@ -1560,12 +1597,12 @@ public class ExpressionImage {
 	try {
  if      (!transmitPrimitive(new Long(1L),new Long(1L))) t.testFail();
  else if (!transmitPrimitive(new Double(1.0d),new Double(1.0d))) t.testFail();
- else if (!transmitPrimitive(new Integer(1),new Long(1L))) t.testFail();
- else if (!transmitPrimitive(new Byte((byte)1),new Long(1L))) t.testFail();
- else if (!transmitPrimitive(new Character(' '),new Long(32L))) t.testFail();
- else if (!transmitPrimitive(new Short((short)-3),new Long(-3L))) t.testFail();
- else if (!transmitPrimitive(new Boolean(true),new Long(1))) t.testFail();
- else if (!transmitPrimitive(new Float(1.0f),new Double(1.0d))) t.testFail();
+ else if (!transmitPrimitive(new Integer(1),new Integer(1))) t.testFail();
+ else if (!transmitPrimitive(new Byte((byte)1),new Byte((byte)1))) t.testFail();
+ else if (!transmitPrimitive(new Character(' '),new Character(' '))) t.testFail();
+ else if (!transmitPrimitive(new Short((short)-3),new Short((short)-3))) t.testFail();
+ else if (!transmitPrimitive(new Boolean(true),new Boolean(true))) t.testFail();
+ else if (!transmitPrimitive(new Float(1.0f),new Float(1.0f))) t.testFail();
  else t.testOK();
 	} catch (java.lang.Throwable e) {
 	  Debug.reportThrowable(e);
