@@ -1,5 +1,5 @@
 /*
- * $Id: Optimizer.java,v 1.1.1.1 1998/05/11 18:43:02 metlov Exp $
+ * $Id: Optimizer.java,v 1.2 1998/05/23 12:34:50 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -148,27 +148,51 @@ public class Optimizer {
   };
   
   /**
-   * Generates a "convert type" operation.
+   * Generates an explicit "convert type" operation.
    * <P> It has the sense to call this function only for narrowing 
    * conversions (see &sect;5.1.3 and &sect;5.1.5 of the Java Language
    * Specification (JLS) ). 
    * Widening ( &sect;5.1.2,  &sect;5.1.4 of JLS) type conversions are 
    * performed by this compiler automatically.
    * @param to is the class to convert to.
+   * @param widening if true prohibits doing narrowing conversions.
+   * @exception IllegalStateException if requested conversion is not supported.
+   */
+  public void convert(Class to,boolean widening) throws IllegalStateException {
+    seeNonFinished();
+    Class ct=(Class)types.peek();
+    
+    if (!ExpressionImage.canConvert(ct,to))
+      throw new IllegalStateException("Can not convert "+ct.toString()+
+				      " to "+to.toString()+
+				      ".");
+    
+    if (widening && !ExpressionImage.canConvertByWidening(ct,to))
+      throw new IllegalStateException("You must specify narrowing conversion"+
+				      " from "+ct.toString()+" to "+
+				      to.toString()+" explicitly");
+    
+    if (ct!=to) { // need convert
+      OP lastOP=code.getLast();
+      if (lastOP instanceof OP_convert) { // can patch
+	((OP_convert)lastOP).setType(to);
+      } else 
+	code.addLast(new OP_convert(to));
+    };
+    types.pop();
+    types.push(to);
+  };
+
+  /**
+   * Generates an explicit "convert type" operation.
+   * <P> This function is equivalent to <TT>convert(to,false)</TT>.
+   * Widening ( &sect;5.1.2,  &sect;5.1.4 of JLS) type conversions are 
+   * performed by this compiler automatically.
+   * @param to is the class to convert to.
    * @exception IllegalStateException if requested conversion is not supported.
    */
   public void convert(Class to) throws IllegalStateException {
-    seeNonFinished();
-    Class ct=(Class)types.peek();
-
-    if (!ExpressionImage.canConvert(ct,to)) {
-      throw new IllegalStateException("Requested conversion ("+ct.toString()+
-				      " --> "+to.toString()+
-				      ") is not supported.");
-    };
-    code.addLast(new OP_convert(to));
-    types.pop();
-    types.push(to);
+    convert(to,false);
   };
   
   
